/* Autogenerated by ../bin/halcompile on Fri Apr 26 19:11:59 2024 -- do not edit */
#include "rtapi.h"
#ifdef RTAPI
#include "rtapi_app.h"
#endif
#include "rtapi_string.h"
#include "rtapi_errno.h"
#include "hal.h"
#include "rtapi_math64.h"

static int comp_id;

#ifdef MODULE_INFO
MODULE_INFO(linuxcnc, "component:ohmic:LinuxCNC HAL component that uses a Mesa THCAD for ohmic sensing");
MODULE_INFO(linuxcnc, "descr:\nMesa THCAD Card component to scale input and outputs from the Mesa THCAD2, THCAD5, THCAD10, and THCAD300 cards.\n.br\nAllows user configurable voltage thresholds for ohmic sensing.\n\nOutput pins are provided for:\n.br\nohmic-volts -the voltage sensed on ohmic sensing.\n.br\nthcad-volts -the actual voltage measured by the THCAD.\n.br\nohmic-on    -true if ohmic-volts >= ohmic-threshold, false if ohmic-volts <= ohmic-low.\n\nA THCAD-5 would often be used for ohmic sensing in conjunction with a 24 Volt isolated power supply and a 390 kΩ series resistor resulting in a voltage divider of 4.9.\n.br\nThis would result in a full scale reading of 24.5 Volts which is above the power supply output voltage.\n.br\nThe circuit will remain protected by the THCAD's ability to tolerate a 500 Volt over-voltage indefinitely.\n.br\nIt is optional that power to the ohmic sensing circuit be disconnected unless probing is in progress.\n");
MODULE_INFO(linuxcnc, "examples:\n.br\nThe below HAL example assumes a THCAD5 card using a 1/32 frequency setting and a voltage divider internal to the plasma cutter with range extended to 24.5 volts by a series 390K external resistor as per the manual.\nAdditional information and wiring diagram is contained in the Plasma Primer in the LinuxCNC documentation.\n.br\nExample Calibration Data: 0V = 122.9 kHz, 10V = 925.7 kHz should be entered as 122900 and 925700.\n\nloadrt ohmic names=ohmicsense\n.br\naddf ohmicsense servo-thread\n.br\nsetp ohmicsense.thcad-0-volt-freq    122900\n.br\nsetp ohmicsense.thcad-max-volt-freq  925700\n.br\nsetp ohmicsense.thcad-divide         32\n.br\nsetp ohmicsense.thcad-fullscale      5\n.br\nsetp ohmicsense.volt-divider         4.9\n.br\nsetp ohmicsense.threshold            22\n.br\nsetp ohmicsense.ohmic-low            21\n.br\nnet ohmic-vel    ohmicsense.velocity-in <= hm2_7i76e.0.encoder.00.velocity\n.br\nnet ohmic-enable ohmicsense.is_probing  <= plasmac.ohmic-enable\n.br\nnet ohmic-true   ohmicsense.ohmic-on    => plasmac.ohmic-probe\n");
MODULE_INFO(linuxcnc, "author:Rod Webster");
MODULE_INFO(linuxcnc, "pin:is_probing:bit:0:in:True if probing:None:None");
MODULE_INFO(linuxcnc, "pin:ohmic_low:float:0:in:The threshold volts below which ohmic sensing is set to be false:21:None");
MODULE_INFO(linuxcnc, "pin:ohmic_threshold:float:0:in:The threshold volts above which ohmic sensing is set to be true:22:None");
MODULE_INFO(linuxcnc, "pin:thcad_0_volt_freq:float:0:in:0 volt calibration data for THCAD card in Hz:None:None");
MODULE_INFO(linuxcnc, "pin:thcad_divide:float:0:in:THCAD divider set by links on THCAD board (1, 32, 64, or 128):32:None");
MODULE_INFO(linuxcnc, "pin:thcad_fullscale:float:0:in:THCAD full scale in Volt (5, 10, or 300 Volt):5:None");
MODULE_INFO(linuxcnc, "pin:thcad_max_volt_freq:float:0:in:Full scale calibration data for THCAD Card in Hz:None:None");
MODULE_INFO(linuxcnc, "pin:velocity_in:float:0:in:The velocity returned from the THCAD and read by the Mesa encoder input:None:None");
MODULE_INFO(linuxcnc, "pin:volt_divider:float:0:in:The divide ratio:4.9:None");
MODULE_INFO(linuxcnc, "pin:ohmic_on:bit:0:out:True if ohmic circuit is closed (material is sensed):None:None");
MODULE_INFO(linuxcnc, "pin:ohmic_volts:float:0:out:Calculated ohmic voltage:None:None");
MODULE_INFO(linuxcnc, "funct:_:1:");
MODULE_INFO(linuxcnc, "license:GPL");
MODULE_LICENSE("GPL");
#endif // MODULE_INFO


struct __comp_state {
    struct __comp_state *_next;
    hal_bit_t *is_probing;
    hal_float_t *ohmic_low;
    hal_float_t *ohmic_threshold;
    hal_float_t *thcad_0_volt_freq;
    hal_float_t *thcad_divide;
    hal_float_t *thcad_fullscale;
    hal_float_t *thcad_max_volt_freq;
    hal_float_t *velocity_in;
    hal_float_t *volt_divider;
    hal_bit_t *ohmic_on;
    hal_float_t *ohmic_volts;
};
struct __comp_state *__comp_first_inst=0, *__comp_last_inst=0;

static void _(struct __comp_state *__comp_inst, long period);
static int __comp_get_data_size(void);
#undef TRUE
#define TRUE (1)
#undef FALSE
#define FALSE (0)
#undef true
#define true (1)
#undef false
#define false (0)

static int export(char *prefix, long extra_arg) {
    char buf[HAL_NAME_LEN + 1];
    int r = 0;
    int sz = sizeof(struct __comp_state) + __comp_get_data_size();
    struct __comp_state *inst = hal_malloc(sz);
    memset(inst, 0, sz);
    r = hal_pin_bit_newf(HAL_IN, &(inst->is_probing), comp_id,
        "%s.is-probing", prefix);
    if(r != 0) return r;
    r = hal_pin_float_newf(HAL_IN, &(inst->ohmic_low), comp_id,
        "%s.ohmic-low", prefix);
    if(r != 0) return r;
    *(inst->ohmic_low) = 21;
    r = hal_pin_float_newf(HAL_IN, &(inst->ohmic_threshold), comp_id,
        "%s.ohmic-threshold", prefix);
    if(r != 0) return r;
    *(inst->ohmic_threshold) = 22;
    r = hal_pin_float_newf(HAL_IN, &(inst->thcad_0_volt_freq), comp_id,
        "%s.thcad-0-volt-freq", prefix);
    if(r != 0) return r;
    r = hal_pin_float_newf(HAL_IN, &(inst->thcad_divide), comp_id,
        "%s.thcad-divide", prefix);
    if(r != 0) return r;
    *(inst->thcad_divide) = 32;
    r = hal_pin_float_newf(HAL_IN, &(inst->thcad_fullscale), comp_id,
        "%s.thcad-fullscale", prefix);
    if(r != 0) return r;
    *(inst->thcad_fullscale) = 5;
    r = hal_pin_float_newf(HAL_IN, &(inst->thcad_max_volt_freq), comp_id,
        "%s.thcad-max-volt-freq", prefix);
    if(r != 0) return r;
    r = hal_pin_float_newf(HAL_IN, &(inst->velocity_in), comp_id,
        "%s.velocity-in", prefix);
    if(r != 0) return r;
    r = hal_pin_float_newf(HAL_IN, &(inst->volt_divider), comp_id,
        "%s.volt-divider", prefix);
    if(r != 0) return r;
    *(inst->volt_divider) = 4.9;
    r = hal_pin_bit_newf(HAL_OUT, &(inst->ohmic_on), comp_id,
        "%s.ohmic-on", prefix);
    if(r != 0) return r;
    r = hal_pin_float_newf(HAL_OUT, &(inst->ohmic_volts), comp_id,
        "%s.ohmic-volts", prefix);
    if(r != 0) return r;
    rtapi_snprintf(buf, sizeof(buf), "%s", prefix);
    r = hal_export_funct(buf, (void(*)(void *inst, long))_, inst, 1, 0, comp_id);
    if(r != 0) return r;
    if(__comp_last_inst) __comp_last_inst->_next = inst;
    __comp_last_inst = inst;
    if(!__comp_first_inst) __comp_first_inst = inst;
    return 0;
}
static int default_count=1, count=0;
RTAPI_MP_INT(count, "number of ohmic");
char *names = ""; // comma separated names
RTAPI_MP_STRING(names, "names of ohmic");
int rtapi_app_main(void) {
    int r = 0;
    int i;
    comp_id = hal_init("ohmic");
    if(comp_id < 0) return comp_id;
    if(count && names[0]) {
        rtapi_print_msg(RTAPI_MSG_ERR,"count= and names= are mutually exclusive\n");
        return -EINVAL;
    }
    if(!count && !names[0]) count = default_count;
    if(count) {
        for(i=0; i<count; i++) {
            char buf[HAL_NAME_LEN + 1];
            rtapi_snprintf(buf, sizeof(buf), "ohmic.%d", i);
            r = export(buf, i);
            if(r != 0) break;
       }
    } else {
        size_t i, j;
        int idx;
        char buf[HAL_NAME_LEN+1];
        const size_t length = strlen(names);
        for (i = j = idx = 0; i <= length; i++) {
            const char c = buf[j] = names[i];
            if ((c == ',') || (c == '\0')) {
                buf[j] = '\0';
                r = export(buf, idx);
                if(r != 0) {break;}
                idx++;
                j = 0;
            } else {
                if (++j == (sizeof(buf) / sizeof(buf[0]))) {
                    buf[j - 1] = '\0';
                    rtapi_print_msg(RTAPI_MSG_ERR,"names: \"%s\" too long\n", buf);
                    r = -EINVAL;
                    break;
                }
            }
        }
    }
    if(r) {
        hal_exit(comp_id);
    } else {
        hal_ready(comp_id);
    }
    return r;
}

void rtapi_app_exit(void) {
    hal_exit(comp_id);
}

#undef FUNCTION
#define FUNCTION(name) static void name(struct __comp_state *__comp_inst, long period)
#undef EXTRA_SETUP
#define EXTRA_SETUP() static int extra_setup(struct __comp_state *__comp_inst, char *prefix, long extra_arg)
#undef EXTRA_CLEANUP
#define EXTRA_CLEANUP() static void extra_cleanup(void)
#undef fperiod
#define fperiod (period * 1e-9)
#undef is_probing
#define is_probing (0+*__comp_inst->is_probing)
#undef ohmic_low
#define ohmic_low (0+*__comp_inst->ohmic_low)
#undef ohmic_threshold
#define ohmic_threshold (0+*__comp_inst->ohmic_threshold)
#undef thcad_0_volt_freq
#define thcad_0_volt_freq (0+*__comp_inst->thcad_0_volt_freq)
#undef thcad_divide
#define thcad_divide (0+*__comp_inst->thcad_divide)
#undef thcad_fullscale
#define thcad_fullscale (0+*__comp_inst->thcad_fullscale)
#undef thcad_max_volt_freq
#define thcad_max_volt_freq (0+*__comp_inst->thcad_max_volt_freq)
#undef velocity_in
#define velocity_in (0+*__comp_inst->velocity_in)
#undef volt_divider
#define volt_divider (0+*__comp_inst->volt_divider)
#undef ohmic_on
#define ohmic_on (*__comp_inst->ohmic_on)
#undef ohmic_volts
#define ohmic_volts (*__comp_inst->ohmic_volts)


#line 75 "hal/components/ohmic.comp"
#include <rtapi_math.h>

FUNCTION(_) {
    double thcad_vel_scale = 1/((thcad_max_volt_freq - thcad_0_volt_freq)/thcad_fullscale/thcad_divide);
    double thcad_scale_offset = thcad_0_volt_freq/thcad_divide;

    ohmic_volts = (velocity_in - thcad_scale_offset) * thcad_vel_scale * volt_divider;
    if(is_probing)
        ohmic_on = (ohmic_volts >= ohmic_threshold ? 1 : 0);
    else
        if(ohmic_volts <= ohmic_low)
            ohmic_on = 0;
}

static int __comp_get_data_size(void) { return 0; }
